Index: formats/format_wav.c
===================================================================
RCS file: /usr/cvsroot/asterisk/formats/format_wav.c,v
retrieving revision 1.13
diff -u -r1.13 format_wav.c
--- formats/format_wav.c	8 Sep 2003 16:48:07 -0000	1.13
+++ formats/format_wav.c	14 Nov 2003 05:01:35 -0000
@@ -128,7 +128,7 @@
 		ast_log(LOG_WARNING, "Read failed (formtype)\n");
 		return -1;
 	}
-	if (ltohl(hsize) != 16) {
+	if (ltohl(hsize) < 16) {
 		ast_log(LOG_WARNING, "Unexpected header size %d\n", ltohl(hsize));
 		return -1;
 	}
@@ -174,27 +174,25 @@
 		ast_log(LOG_WARNING, "Read failed (Bits Per Sample): %d\n", ltohs(bisam));
 		return -1;
 	}
-        /* Begin data chunk */
-	if (read(fd, &data, 4) != 4) {
-		ast_log(LOG_WARNING, "Read failed (data)\n");
-		return -1;
-	}
-	if (memcmp(&data, "data", 4)) {
-		ast_log(LOG_WARNING, "Does not say data\n");
-		return -1;
-	}
-	/* Data has the actual length of data in it */
-	if (read(fd, &data, 4) != 4) {
-		ast_log(LOG_WARNING, "Read failed (data)\n");
-		return -1;
+	if(ltohl(hsize) > 16)
+		lseek(fd, ltohl(hsize)-16, SEEK_CUR);
+	for(;;)
+	{
+	        /* Begin data chunk */
+		if (read(fd, &data, 4) != 4) {
+			ast_log(LOG_WARNING, "Read failed (data)\n");
+			return -1;
+		}
+		/* Data has the actual length of data in it */
+		if (read(fd, &hsize, 4) != 4) {
+			ast_log(LOG_WARNING, "Read failed (data)\n");
+			return -1;
+		}
+		if (!memcmp(&data, "data", 4))
+			break;
+		lseek(fd, ltohl(hsize), SEEK_CUR);
 	}
-#if 0
-	curpos = lseek(fd, 0, SEEK_CUR);
-	truelength = lseek(fd, 0, SEEK_END);
-	lseek(fd, curpos, SEEK_SET);
-	truelength -= curpos;
-#endif	
-	return ltohl(data);
+	return ltohl(hsize);
 }
 
 static int update_header(int fd)
@@ -378,6 +376,18 @@
 	s = NULL;
 }
 
+static inline void wav_autogain(struct ast_filestream *s, short *tmp, int samples)
+{
+	for (x = samples; x-- > 0; ) {
+		if (tmp[x] & ((1 << GAIN) - 1)) {
+			/* If it has data down low, then it's not something we've artificially increased gain
+			   on, so we don't need to gain adjust it */
+			s->needsgain = 0;
+			break;
+		}
+	}
+}
+
 static struct ast_frame *wav_read(struct ast_filestream *s, int *whennext)
 {
 	int res;
@@ -405,14 +415,8 @@
 	for( x = 0; x < sizeof(tmp)/2; x++) tmp[x] = (tmp[x] << 8) | ((tmp[x] & 0xff00) >> 8);
 #endif
 
-	if (s->needsgain) {
-		for (x=0;x<sizeof(tmp)/2;x++)
-			if (tmp[x] & ((1 << GAIN) - 1)) {
-				/* If it has data down low, then it's not something we've artificially increased gain
-				   on, so we don't need to gain adjust it */
-				s->needsgain = 0;
-			}
-	}
+	if (s->needsgain)
+		wav_autogain(s, tmp, sizeof(tmp)/2);
 	if (s->needsgain) {
 		for (x=0;x<sizeof(tmp)/2;x++) {
 			s->buf[x] = tmp[x] >> GAIN;
@@ -460,21 +464,23 @@
 
 	if (fs->buf) {
 		tmpi = f->data;
+
+		if (fs->needsgain)
+			wav_autogain(fs, tmpi, f->datalen / 2);
+
 		/* Volume adjust here to accomodate */
-		for (x=0;x<f->datalen/2;x++) {
-			tmpf = ((float)tmpi[x]) * ((float)(1 << GAIN));
-			if (tmpf > 32767.0)
-				tmpf = 32767.0;
-			if (tmpf < -32768.0)
-				tmpf = -32768.0;
-			tmp[x] = tmpf;
-			tmp[x] &= ~((1 << GAIN) - 1);
+		if (fs->needsgain) {
+			for (x=0;x<f->datalen/2;x++) {
+				tmp[x] = tmpi[x] << GAIN;
+			}
+		} else
+			memcpy(tmp, tmpi, f->datalen);
 
 #if __BYTE_ORDER == __BIG_ENDIAN
+		for (x=0;x<f->datalen/2;x++)
 			tmp[x] = (tmp[x] << 8) | ((tmp[x] & 0xff00) >> 8);
 #endif
 
-		}
 		if ((write (fs->fd, tmp, f->datalen) != f->datalen) ) {
 			ast_log(LOG_WARNING, "Bad write (%d): %s\n", res, strerror(errno));
 			return -1;
